ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
c                                                                     c
c  This program is used to append grads grid data at a single         c
c  pressure level to an external file containing grads data from      c
c  multiple pressure levels.  Each call to the UDF (which calls this  c
c  function ) appends the standard grid header data first, then       c
c  appends the actual grid data to the end of the file.  When used    c
c  in a loop, theis effectively allows you to write out three         c
c  dimensions of grads data even though the UDF feature only gives    c
c  you to data at a fixed pressure level.                             c
c                                                                     c
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      PROGRAM apendgrd

      IMPLICIT NONE

      INTEGER, PARAMETER :: iunit_gr_out=21, iunit_gr_in=22, 
     & iunit_gr_append=23

      REAL ivals(20), ovals(20)
      REAL, ALLOCATABLE :: grid_data(:)
      INTEGER nxdim, nydim, nsiz
      INTEGER i

c  Open the transfer files and the append file

      CALL open_files(iunit_gr_out,iunit_gr_in,iunit_gr_append)

c  Load the grid header information from the grads output file

      CALL load_head(iunit_gr_out,ovals)

c  Get the essential dimension informatio from the header and allocate
c  memory for the grid data

      nxdim = nint(ovals(4))
      nydim = nint(ovals(5))
      nsiz  = nxdim * nydim

      ALLOCATE(grid_data(nsiz))

c  Store grid data into memory

      READ(iunit_gr_out) grid_data

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  This section just prepares dummy data to write to the grads
c  input file.

c  Copy grid header from grads output to grid header for grads input

      DO i=1,20
        ivals(i) = ovals(i)
      ENDDO
      
c  Write out dummy data to function result file

      CALL write_frf(iunit_gr_in,ivals,grid_data,nsiz)

c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
            
c  Append the 2D grid data from the current level to the file
c  determined by the first argument

      WRITE(iunit_gr_append) ivals
      WRITE(iunit_gr_append) grid_data

      DEALLOCATE(grid_data)

      CALL close_files(iunit_gr_out,iunit_gr_in,iunit_gr_append)

      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      SUBROUTINE open_files(iunit_gr_out,iunit_gr_in,iunit_gr_append)
  
      IMPLICIT NONE

      INTEGER iunit_gr_out, iunit_gr_in, iunit_gr_append
      INTEGER nargs, dtffrmt
      REAL vals(20)
      CHARACTER(LEN=32) file
      CHARACTER(LEN=41) dirfile

c  Open data transfer files

      OPEN (iunit_gr_out,FILE='./udfs/udf.apndgrd.out',STATUS='unknown',
     &      FORM='unformatted',ACCESS='sequential',POSITION='rewind',
     &      ERR=910)
           
      OPEN (iunit_gr_in,FILE='./udfs/udf.apndgrd.in',STATUS='unknown',
     &      FORM='unformatted',ACCESS='sequential',ERR=920)

      GOTO 400

  910 PRINT *, '!!! ERROR: unable to open udf.apendgrd.out'
      STOP 50

  920 PRINT *, '!!! ERROR: unable to open udf.apendgrd.in'
      STOP 50

  400 CONTINUE

c  Read data transfer file header

      READ(iunit_gr_out) vals

c  Make sure number of arguments is 2

      nargs = nint(vals(1))
      IF (nargs /= 2) THEN
        PRINT *, '!!! ERROR: incorrect number of arguments'
        STOP 50
      ENDIF
      
c  Check to make sure the data transfer file is the correct format

      dtffrmt = nint(vals(2))
      IF (dtffrmt /= 0) THEN
        PRINT *, '!!! ERROR: incorrect file transfer format'
	STOP 50
      ENDIF

c  Read the name of the file

      READ(iunit_gr_out) file
      dirfile = './udfs/'//file

c  Open the specified file

      OPEN (iunit_gr_append,FILE=dirfile,STATUS='unknown',
     &      FORM='unformatted',ACCESS='sequential',
     &      POSITION='append',ERR=930)

      GOTO 500

  930 PRINT *, '!!! ERROR: unable to open '//file
      STOP 50

  500 CONTINUE

      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      SUBROUTINE load_head(iunit_gr_out,ovals)
      
      IMPLICIT NONE

      INTEGER iunit_gr_out
      INTEGER idim, jdim
      REAL ovals(20)
      REAL vals(20)
            
      READ(iunit_gr_out) ovals

      idim = nint(ovals(2))
      jdim = nint(ovals(3))
            
c  Check to make sure the i and j dimensions are X and Y respectively
      
      IF (idim /= 0) THEN
        PRINT *, '!!! ERROR: first dimenstion must be X'
        STOP 50
      ENDIF
      IF (jdim /= 1) THEN
        PRINT *, '!!! ERROR: second dimension must be Y'
        STOP 50
      ENDIF
    
      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      SUBROUTINE write_frf(iunit_gr_in,ivals,grid_data,nsiz)

c  This just writes dummy data to the function result file

      IMPLICIT NONE

      INTEGER iunit_gr_in
      INTEGER nsiz
      REAL vals(20), ivals(20), grid_data(nsiz)

      vals(1) = 0.0
      vals(2) = 0.0

      WRITE(iunit_gr_in) vals
      WRITE(iunit_gr_in) ivals
      WRITE(iunit_gr_in) grid_data

      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



      SUBROUTINE close_files(iunit_gr_out,iunit_gr_in,iunit_gr_append)

      IMPLICIT NONE

      INTEGER iunit_gr_out, iunit_gr_in, iunit_gr_append
      
      CLOSE(iunit_gr_out)
      CLOSE(iunit_gr_in)
      CLOSE(iunit_gr_append)

      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

