ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
c                                                                     c
c This program takes a cylindrical grid file created with the UDF     c
c rec2cyl, averages all the values at each radius, and writes the     c
c data to the function result file so that grads can plot the         c
c azimuthal average using the display command.                        c
c                                                                     c
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      PROGRAM cyl2aza

      IMPLICIT NONE

      INTEGER, PARAMETER :: iunit_gr_out=21, iunit_gr_in=22,
     & iunit_gr_cyl=23

      REAL ovals(20), ivals(20), cyl_header(10)
      REAL, ALLOCATABLE :: cyl_data(:,:,:)
      REAL startaz, endaz, range, botlev, toplev
      INTEGER nazimuth,nrange,nlevs
      INTEGER i

c  Open the files

      CALL open_files(iunit_gr_out,iunit_gr_in,
     &                iunit_gr_cyl)

c  Load the grid header information from the grads output file

      READ(iunit_gr_out) ovals

c  Loat the grid header information from the cylindrical grid file
      
      READ(iunit_gr_cyl) cyl_header
      
      nazimuth = nint(cyl_header(3))
      nrange   = nint(cyl_header(4))
      nlevs    = nint(cyl_header(5))
      botlev   = cyl_header(9)
      toplev   = cyl_header(10)

c  Fill in header information for the grads input file

      DO i=1,20
        ivals(i) = ovals(i)
      ENDDO

      ivals(2)  = 0.0
      ivals(3)  = 2.0
      ivals(4)  = nrange + 0.0
      ivals(5)  = nlevs + 0.0
      ivals(6)  = 1.0
      ivals(7)  = 1.0
      ivals(10) = botlev
      ivals(11) = int((toplev-botlev)/(nlevs-1)) + 0.0

c  Allocate cylindracal grid data array and read from file
      
      ALLOCATE(cyl_data(nazimuth,nrange,nlevs))

      READ(iunit_gr_cyl) cyl_data

c  Write out data to the function result file for GrADS

      CALL write_frf(iunit_gr_in,ivals,cyl_data,
     &               nazimuth,nrange,nlevs)

c  Deallocata all arrays and close all open files

      DEALLOCATE(cyl_data)

      CALL close_files(iunit_gr_out,iunit_gr_in,
     &                 iunit_gr_cyl)

      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      SUBROUTINE open_files(iunit_gr_out,iunit_gr_in,
     &                      iunit_gr_cyl)
  
      IMPLICIT NONE

      INTEGER iunit_gr_out, iunit_gr_in, iunit_gr_cyl
      INTEGER nargs, dtffrmt
      REAL vals(20)
      CHARACTER(LEN=32) ifile
      CHARACTER(LEN=32) ofile
      CHARACTER(LEN=41) dirfile

c  Open data transfer files

      OPEN (iunit_gr_out,FILE='./udfs/udf.cyl2aza.out',
     &      STATUS='unknown', FORM='unformatted',ACCESS='sequential',
     &      POSITION='rewind',ERR=910)
           
      OPEN (iunit_gr_in,FILE='./udfs/udf.cyl2aza.in',STATUS='unknown',
     &      FORM='unformatted',ACCESS='sequential',ERR=920)

      GOTO 400

  910 PRINT *, '!!! ERROR: unable to open udf.cyl2aza.out'
      STOP 50

  920 PRINT *, '!!! ERROR: unable to open udf.cyl2aza.in'
      STOP 50

  400 CONTINUE

c  Read data transfer file header

      READ(iunit_gr_out) vals

c  Make sure number of arguments is 2

      nargs = nint(vals(1))
      IF (nargs /= 2) THEN
        PRINT *, '!!! ERROR: incorrect number of arguments'
        STOP 50
      ENDIF
      
c  Check to make sure the data transfer file is the correct format

      dtffrmt = nint(vals(2))
      IF (dtffrmt /= 0) THEN
        PRINT *, '!!! ERROR: incorrect file transfer format'
	STOP 50
      ENDIF

c  Read the name of the cylindrical grid file

      READ(iunit_gr_out) ifile
      dirfile = './udfs/'//ifile

c  Open the specified file

      OPEN (iunit_gr_cyl,FILE=dirfile,STATUS='old',
     &      FORM='unformatted',ACCESS='sequential',
     &      POSITION='rewind',ERR=930)

      GOTO 500

  930 PRINT *, '!!! ERROR: unable to open '//ifile
      STOP 50

  500 CONTINUE

      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      SUBROUTINE write_frf(iunit_gr_in,ivals,cyl_data,
     &                     nazimuth,nrange,nlevs)

c  This function averages the data around each ring at each level and 
c  stores the result in a 2D grid that GrADS can display.

      IMPLICIT NONE

      INTEGER iunit_gr_in
      INTEGER nazimuth,nrange,nlevs,i,j,k
      REAL vals(20), ivals(20)
      REAL cyl_data(nazimuth,nrange,nlevs)
      REAL, ALLOCATABLE :: ave_data(:,:)
      REAL sum

      ALLOCATE(ave_data(nrange,nlevs))

      DO i=1,nrange
        DO j=1,nlevs
          sum = 0.0
          DO k=1,nazimuth
            sum = sum + cyl_data(k,i,j)
          ENDDO
          ave_data(i,j) = sum / nazimuth
        ENDDO
      ENDDO

      vals(1) = 0.0
      vals(2) = 0.0

      WRITE(iunit_gr_in) vals
      WRITE(iunit_gr_in) ivals
      WRITE(iunit_gr_in) ave_data

      DEALLOCATE(ave_data)

      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      SUBROUTINE close_files(iunit_gr_out,iunit_gr_in,
     &                       iunit_gr_cyl)

c  Close all files opened in program

      IMPLICIT NONE

      INTEGER iunit_gr_out, iunit_gr_in, iunit_gr_cyl
      
      CLOSE(iunit_gr_out)
      CLOSE(iunit_gr_in)
      CLOSE(iunit_gr_cyl)

      RETURN
      END

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c---------------------------------------------------------------------c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
